@@ -5,13 +5,15 @@ Ported from https://github.com/philomena-dev/philomena/blob/master/assets/js/mat
 """
 
 import abc
+import dateutil.parser
 import re
+import Levenshtein
 import math
 import time
 
 from datetime import datetime, timezone, timedelta
 from dateutil.relativedelta import relativedelta
-from typing import cast, Callable, List, Optional, Sequence, Tuple, Union
+from typing import cast, Callable, Iterable, List, Optional, Sequence, Tuple, Union
 
 from . import api, utils
 from .types import Image, Interaction
@@ -134,8 +136,11 @@ class SearchTerm(SearchOperand):
             re.compile(r"^:(\d{2})"),
         ]
         
+        if not date_val:
+            raise ParseError("Empty term")
+        
         timezone_offset = [0, 0]
-        time_data = [0, 0, 1, 0, 0, 0]
+        time_data = [0, 1, 1, 0, 0, 0]
         orig_date_val = date_val
         local_date_val = date_val
         
@@ -154,6 +159,7 @@ class SearchTerm(SearchOperand):
         
         for i, regex in enumerate(PARSE_RES):
             if not local_date_val:
+                i -= 1
                 break
             
             match = regex.match(local_date_val)
@@ -285,32 +291,9 @@ class SearchTerm(SearchOperand):
         else:
             target_distance = 0
         
-        target_str_lower = target_str.lower()
+        distance = Levenshtein.distance(self.term, target_str.lower())
         
-        # Work vectors, representing the last three populated
-        # rows of the dynamic programming matrix of the iterative
-        # optimal string alignment calculation.
-        v0: List[int] = []
-        v1: List[int] = list(range(len(target_str_lower) + 1))
-        v2: List[int] = []
-        
-        for i, term_c in enumerate(self.term):
-            v2[0] = i
-            for j, target_c in enumerate(target_str_lower):
-                cost = 0 if term_c == target_c else 1
-                v2[j + 1] = min(
-                    v1[j + 1] + 1,  # deletion
-                    v2[j] + 1,      # insertion
-                    v1[j] + cost,
-                )
-                if i > 1 and j > 1 and term_c == target_str_lower[j - 1] and \
-                        target_str_lower[i - 1] == target_c:
-                    v2[j + 1] = min(v2[j], v0[j - 1] + cost)
-            
-            # Rotate dem vec pointers bra.
-            v0, v1, v2 = v1, v2, v0
-        
-        return v1[len(target_str_lower)] < target_distance
+        return distance <= target_distance
     
     def exact_match(self, target_str: str) -> bool:
         if not isinstance(self.term, str):
@@ -330,7 +313,7 @@ class SearchTerm(SearchOperand):
         type: str,
         interaction: Optional[str],
         interactions: Sequence[Interaction],
-    ):
+    ) -> bool:
         return any(
             v.image_id == image_id and v.interaction_type == type and (interaction is None or v.value == interaction)
             for v in interactions
@@ -372,7 +355,7 @@ class SearchTerm(SearchOperand):
             return False
         
         if self.term_type == "date":
-            date = datetime.fromisoformat(getattr(target, self.term_space))
+            date = dateutil.parser.isoparse(getattr(target, self.term_space))
             # The open-left, closed-right date range specified by the
             # date/time format limits the types of comparisons that are
             # done compared to numeric ranges.
@@ -442,7 +425,7 @@ class OrOperator(SearchOperand):
         return self.op1.match(target, interactions) or self.op2.match(target, interactions)
 
 
-def generate_lex_array(search_str) -> List[Union[SearchTerm, str]]:
+def generate_lex_array(search_str: str) -> List[Union[SearchTerm, str]]:
     op_queue: List[str] = []
     group_negate: List[bool] = []
     token_stack: List[Union[SearchTerm, str]] = []
@@ -607,7 +590,7 @@ def parse_search(search: str) -> SearchOperand:
     return parse_tokens(generate_lex_array(search))
 
 
-def main():
+def main() -> None:
     import argparse
     parser = argparse.ArgumentParser()
     group = parser.add_mutually_exclusive_group(required=True)
@@ -617,6 +600,8 @@ def main():
     
     args = parser.parse_args()
     
+    images: Iterable[Image]
+    
     site = api.Site("https://derpibooru.org/")
     if args.query:
         images = site.search_images(args.query)
